'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _fsPromise = require('fs-promise');

var _fsPromise2 = _interopRequireDefault(_fsPromise);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _ScreenshotStrategyManager = require('../utils/ScreenshotStrategyManager');

var _ScreenshotStrategyManager2 = _interopRequireDefault(_ScreenshotStrategyManager);

var _getScreenDimensions = require('../scripts/getScreenDimensions');

var _getScreenDimensions2 = _interopRequireDefault(_getScreenDimensions);

var _virtualScroll = require('../scripts/virtualScroll');

var _virtualScroll2 = _interopRequireDefault(_virtualScroll);

var _generateUUID = require('../utils/generateUUID');

var _generateUUID2 = _interopRequireDefault(_generateUUID);

var _saveBase64Image = require('../utils/saveBase64Image');

var _saveBase64Image2 = _interopRequireDefault(_saveBase64Image);

var _image = require('../utils/image');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var log = (0, _debug2.default)('wdio-screenshot:makeAreaScreenshot');
var tmpDir = _path2.default.join(__dirname, '..', '..', '.tmp');

exports.default = function makeAreaScreenshot(browser, startX, startY, endX, endY) {
  var screenDimensions, screenshotStrategy, uuid, dir, cropImages, loop, _screenshotStrategy$g, x, y, indexX, indexY, filePath, base64Screenshot, cropDimensions, croppedBase64Screenshot, mergedBase64Screenshot;

  return _regenerator2.default.async(function makeAreaScreenshot$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          log('requested a screenshot for the following area: %j', { startX: startX, startY: startY, endX: endX, endY: endY });

          _context.next = 3;
          return _regenerator2.default.awrap(browser.execute(_getScreenDimensions2.default));

        case 3:
          screenDimensions = _context.sent.value;

          log('detected screenDimensions %j', screenDimensions);

          screenshotStrategy = _ScreenshotStrategyManager2.default.getStrategy(browser, screenDimensions);

          screenshotStrategy.setScrollArea(startX, startY, endX, endY);

          uuid = (0, _generateUUID2.default)();
          dir = _path2.default.join(tmpDir, uuid);
          _context.prev = 9;
          _context.next = 12;
          return _regenerator2.default.awrap(_fsPromise2.default.ensureDir(dir));

        case 12:
          cropImages = [];
          loop = false;

        case 14:
          _screenshotStrategy$g = screenshotStrategy.getScrollPosition();
          x = _screenshotStrategy$g.x;
          y = _screenshotStrategy$g.y;
          indexX = _screenshotStrategy$g.indexX;
          indexY = _screenshotStrategy$g.indexY;

          log('scroll to coordinates x: %s, y: %s for index x: %s, y: %s', x, y, indexX, indexY);

          _context.next = 22;
          return _regenerator2.default.awrap(browser.execute(_virtualScroll2.default, x, y));

        case 22:
          _context.next = 24;
          return _regenerator2.default.awrap(browser.pause(100));

        case 24:
          filePath = _path2.default.join(dir, indexY + '-' + indexX + '.png');
          _context.next = 27;
          return _regenerator2.default.awrap(browser.screenshot());

        case 27:
          base64Screenshot = _context.sent.value;
          cropDimensions = screenshotStrategy.getCropDimensions();

          log('crop screenshot with width: %s, height: %s, offsetX: %s, offsetY: %s', cropDimensions.getWidth(), cropDimensions.getHeight(), cropDimensions.getX(), cropDimensions.getY());

          _context.next = 32;
          return _regenerator2.default.awrap((0, _image.cropImage)(base64Screenshot, cropDimensions));

        case 32:
          croppedBase64Screenshot = _context.sent;
          _context.next = 35;
          return _regenerator2.default.awrap((0, _saveBase64Image2.default)(filePath, croppedBase64Screenshot));

        case 35:

          if (!Array.isArray(cropImages[indexY])) {
            cropImages[indexY] = [];
          }

          cropImages[indexY][indexX] = filePath;

          loop = screenshotStrategy.hasNextScrollPosition();
          screenshotStrategy.moveToNextScrollPosition();

        case 39:
          if (loop) {
            _context.next = 14;
            break;
          }

        case 40:

          log('revert scroll to x: %s, y: %s', 0, 0);
          _context.next = 43;
          return _regenerator2.default.awrap(browser.execute(_virtualScroll2.default, 0, 0, true));

        case 43:
          _context.next = 45;
          return _regenerator2.default.awrap((0, _image.mergeImages)(cropImages));

        case 45:
          mergedBase64Screenshot = _context.sent;
          _context.next = 48;
          return _regenerator2.default.awrap(_fsPromise2.default.remove(dir));

        case 48:
          return _context.abrupt('return', mergedBase64Screenshot);

        case 51:
          _context.prev = 51;
          _context.t0 = _context['catch'](9);
          _context.prev = 53;
          _context.next = 56;
          return _regenerator2.default.awrap(_fsPromise2.default.remove(dir));

        case 56:
          _context.next = 60;
          break;

        case 58:
          _context.prev = 58;
          _context.t1 = _context['catch'](53);

        case 60:
          throw _context.t0;

        case 61:
        case 'end':
          return _context.stop();
      }
    }
  }, null, this, [[9, 51], [53, 58]]);
};